@@ -1,9 +1,12 @@
-from .trash import TopTrashDirRules, TrashDir, path_of_backup_copy, print_version, println, Clock
+import collections
+
+from .list import TrashDirsSelector
+from .trash import TopTrashDirRules, TrashDir, path_of_backup_copy, \
+    print_version, println, Clock, parse_deletion_date, trash_dir_found, UserInfoProvider, AllUsersInfoProvider
 from .trash import TrashDirsScanner
 from .trash import EX_OK
 from .trash import PrintHelp
 from .trash import EX_USAGE
-from .trash import ParseTrashInfo
 import os
 import sys
 from trashcli.list_mount_points import os_mount_points
@@ -39,6 +42,15 @@ class Errors:
         self.err.write("%s: %s\n" % (self.program_name, msg))
 
 
+def description(program_name, printer):
+    printer.usage('Usage: %s [days]' % program_name)
+    printer.summary('Purge trashed files.')
+    printer.options(
+       "  --version   show program's version number and exit",
+       "  -h, --help  show this help message and exit")
+    printer.bug_reporting()
+
+
 class EmptyCmd:
     def __init__(self,
                  out,
@@ -54,12 +66,21 @@ class EmptyCmd:
         self.out = out
         self.err = err
         self.file_reader = file_reader
-        self.environ = environ
-        self.getuid = getuid
-        self.list_volumes = list_volumes
         self.version = version
         self.clock = Clock(now, environ)
-        self.file_remover = file_remover
+        file_remover_with_error = FileRemoveWithErrorHandling(file_remover,
+                                                              self.print_cannot_remove_error)
+        self.trashcan = CleanableTrashcan(file_remover_with_error)
+        user_info_provider = UserInfoProvider(environ, getuid)
+        user_dir_scanner = TrashDirsScanner(user_info_provider,
+                                            list_volumes,
+                                            TopTrashDirRules(file_reader))
+        all_users_info_provider = AllUsersInfoProvider()
+        all_users_scanner = TrashDirsScanner(all_users_info_provider,
+                                             list_volumes,
+                                             TopTrashDirRules(file_reader))
+        self.selector = TrashDirsSelector(user_dir_scanner.scan_trash_dirs(),
+                                          all_users_scanner.scan_trash_dirs())
 
     def run(self, *argv):
         program_name = os.path.basename(argv[0])
@@ -71,7 +92,7 @@ class EmptyCmd:
         if result == 'print_version':
             print_version(self.out, program_name, self.version)
         elif result == 'print_help':
-            PrintHelp(self.description, self.out).my_print_help(program_name)
+            PrintHelp(description, self.out).my_print_help(program_name)
         elif result == 'invalid_option':
             invalid_option, = args
             self.errors.print_error("invalid option -- '%s'" % invalid_option)
@@ -80,65 +101,27 @@ class EmptyCmd:
             now_value = self.clock.get_now_value(self.errors)
             println(self.out, now_value.replace(microsecond=0).isoformat())
         elif result == 'default':
-            trash_dirs, arguments, = args
-            self._dustman = DeleteAnything()
-            if len(trash_dirs) > 0:
-                for trash_dir in trash_dirs:
-                    self.empty_trashdir(trash_dir)
+            if not args.max_age_in_days:
+                delete_mode = DeleteAnything()
             else:
-                for argument in arguments:
-                    max_age_in_days = int(argument)
-                    self._dustman = DeleteAccordingDate(self.file_reader.contents_of,
-                                                        self.clock,
-                                                        max_age_in_days,
-                                                        self.errors)
-                self.empty_all_trashdirs()
+                delete_mode = DeleteAccordingDate(self.file_reader.contents_of,
+                                                  self.clock,
+                                                  args.max_age_in_days,
+                                                  self.errors)
+            trash_dirs = self.selector.select(args.all_users,
+                                              args.user_specified_trash_dirs)
+            for event, args in trash_dirs:
+                if event == trash_dir_found:
+                    trash_dir_path, volume = args
+                    trash_dir = TrashDir(self.file_reader)
+                    for trashinfo_path in trash_dir.list_trashinfo(trash_dir_path):
+                        if delete_mode.ok_to_delete(trashinfo_path):
+                            self.trashcan.delete_trashinfo_and_backup_copy(trashinfo_path)
+                    for orphan in trash_dir.list_orphans(trash_dir_path):
+                        self.trashcan.delete_orphan(orphan)
 
         return exit_code
 
-    def description(self, program_name, printer):
-        printer.usage('Usage: %s [days]' % program_name)
-        printer.summary('Purge trashed files.')
-        printer.options(
-           "  --version   show program's version number and exit",
-           "  -h, --help  show this help message and exit")
-        printer.bug_reporting()
-
-    def empty_trashdir(self, specific_dir):
-        self.delete_all_things_under_trash_dir(specific_dir)
-
-    def empty_all_trashdirs(self):
-        scanner = TrashDirsScanner(self.environ,
-                                   self.getuid,
-                                   self.list_volumes,
-                                   TopTrashDirRules(self.file_reader))
-
-        for event, args in scanner.scan_trash_dirs():
-            if event == TrashDirsScanner.Found:
-                path, volume = args
-                self.delete_all_things_under_trash_dir(path)
-
-    def delete_all_things_under_trash_dir(self, trash_dir_path):
-        trash_dir = TrashDir(self.file_reader)
-        for trash_info in trash_dir.list_trashinfo(trash_dir_path):
-            self.delete_trashinfo_and_backup_copy(trash_info)
-        for orphan in trash_dir.list_orphans(trash_dir_path):
-            self.delete_orphan(orphan)
-
-    def delete_trashinfo_and_backup_copy(self, trashinfo_path):
-        trashcan = self.make_trashcan()
-        self._dustman.delete_if_ok(trashinfo_path, trashcan)
-
-    def delete_orphan(self, path_to_backup_copy):
-        trashcan = self.make_trashcan()
-        trashcan.delete_orphan(path_to_backup_copy)
-
-    def make_trashcan(self):
-        file_remover_with_error = FileRemoveWithErrorHandling(self.file_remover,
-                                                              self.print_cannot_remove_error)
-        trashcan = CleanableTrashcan(file_remover_with_error)
-        return trashcan
-
     def print_cannot_remove_error(self, path):
         self.errors.print_error("cannot remove %s" % path)
 
@@ -163,45 +146,31 @@ class FileRemoveWithErrorHandling:
 
 class DeleteAccordingDate:
     def __init__(self, contents_of, clock, max_age_in_days, errors):
-        self._contents_of = contents_of
+        self.contents_of = contents_of
         self.clock = clock
         self.max_age_in_days = max_age_in_days
         self.errors = errors
 
-    def delete_if_ok(self, trashinfo_path, trashcan):
-        contents = self._contents_of(trashinfo_path)
+    def ok_to_delete(self, trashinfo_path):
+        contents = self.contents_of(trashinfo_path)
         now_value = self.clock.get_now_value(self.errors)
-        ParseTrashInfo(
-            on_deletion_date=IfDate(
-                OlderThan(self.max_age_in_days, now_value),
-                lambda: trashcan.delete_trashinfo_and_backup_copy(trashinfo_path)
-            ),
-        )(contents)
+        deletion_date = parse_deletion_date(contents)
+        if deletion_date is not None:
+            if older_than(self.max_age_in_days, now_value, deletion_date):
+                return True
+        return False
 
 
 class DeleteAnything:
 
-    def delete_if_ok(self, trashinfo_path, trashcan):
-        trashcan.delete_trashinfo_and_backup_copy(trashinfo_path)
+    def ok_to_delete(self, _trashinfo_path):
+        return True
 
 
-class IfDate:
-    def __init__(self, date_criteria, then):
-        self.date_criteria = date_criteria
-        self.then = then
-
-    def __call__(self, date2):
-        if self.date_criteria(date2):
-            self.then()
-
-
-class OlderThan:
-    def __init__(self, days_ago, now_value):
-        from datetime import timedelta
-        self.limit_date = now_value - timedelta(days=days_ago)
-
-    def __call__(self, deletion_date):
-        return deletion_date < self.limit_date
+def older_than(days_ago, now_value, deletion_date):
+    from datetime import timedelta
+    limit_date = now_value - timedelta(days=days_ago)
+    return deletion_date < limit_date
 
 
 class CleanableTrashcan:
@@ -217,6 +186,11 @@ class CleanableTrashcan:
         self._file_remover.remove_file(trashinfo_path)
 
 
+Parsed = collections.namedtuple('Parsed', ['all_users',
+                                           'user_specified_trash_dirs',
+                                           'max_age_in_days'])
+
+
 def parse_argv(args):
     from getopt import getopt, GetoptError
 
@@ -224,12 +198,14 @@ def parse_argv(args):
         options, arguments = getopt(args,
                                     'h',
                                     ['help', 'version', 'trash-dir=',
-                                     'print-time'])
+                                     'print-time', 'all-users'])
     except GetoptError as e:
         invalid_option = e.opt
         return 'invalid_option', (invalid_option,)
     else:
         trash_dirs = []
+        max_days = None
+        all_users = False
         for option, value in options:
             if option in ('--help', '-h'):
                 return 'print_help', ()
@@ -239,4 +215,8 @@ def parse_argv(args):
                 return 'print_time', ()
             if option == '--trash-dir':
                 trash_dirs.append(value)
-        return 'default', (trash_dirs, arguments)
+            if option == '--all-users':
+                all_users = True
+        for arg in arguments:
+            max_days = int(arg)
+        return 'default', Parsed(all_users, trash_dirs, max_days)
