@@ -2,8 +2,9 @@
 from __future__ import absolute_import
 
 import datetime
+import pwd
 
-version = '0.21.6.10.1'
+version = '0.21.6.30'
 
 import os
 import logging
@@ -29,7 +30,7 @@ def path_of_backup_copy(trashinfo_path):
     return os.path.join(trash_dir, 'files', basename)
 
 
-def home_trash_dir_path(environ):
+def home_trash_dir_path_from_env(environ):
     if 'XDG_DATA_HOME' in environ:
         return ['%(XDG_DATA_HOME)s/Trash' % environ]
     elif 'HOME' in environ:
@@ -37,8 +38,12 @@ def home_trash_dir_path(environ):
     return []
 
 
+def home_trash_dir_path_from_home(home_dir):
+    return '%s/.local/share/Trash' % home_dir
+
+
 def home_trash_dir(environ, volume_of):
-    paths = home_trash_dir_path(environ)
+    paths = home_trash_dir_path_from_env(environ)
     for path in paths:
         yield path, volume_of(path)
 
@@ -53,40 +58,62 @@ def volume_trash_dir2(volume, getuid):
     yield path, volume
 
 
-class MyEnum:
-    def __init__(self, name):
-        self.name = name
-
+class MyEnum(str):
     def __repr__(self):
-        return self.name
+        return str(self)
+
+
+trash_dir_found = MyEnum('trash_dir_found')
+trash_dir_skipped_because_parent_not_sticky = \
+    MyEnum('trash_dir_skipped_because_parent_not_sticky')
+trash_dir_skipped_because_parent_is_symlink = \
+    MyEnum('trash_dir_skipped_because_parent_is_symlink')
+
+
+class UserInfo:
+    def __init__(self, home_trash_dir_paths, uid):
+        self.home_trash_dir_paths = home_trash_dir_paths
+        self.uid = uid
+
+
+class UserInfoProvider:
+    def __init__(self, environ, getuid):
+        self.environ = environ
+        self.getuid = getuid
+
+    def get_user_info(self):
+        return [UserInfo(home_trash_dir_path_from_env(self.environ),
+                         self.getuid())]
+
+
+class AllUsersInfoProvider:
+    def get_user_info(self):
+        for user in pwd.getpwall():
+            yield UserInfo([home_trash_dir_path_from_home(user.pw_dir)],
+                           user.pw_uid)
 
 
 class TrashDirsScanner:
-    Found = MyEnum('TrashDirsScanner.Found')
-    SkippedBecauseParentNotSticky = MyEnum('TrashDirsScanner.SkippedBecauseParentNotSticky')
-    SkippedBecauseParentIsSymlink = MyEnum('TrashDirsScanner.SkippedBecauseParentIsSymlink')
-
-    def __init__(self, environ, getuid, list_volumes, top_trash_dir_rules):
-        self.getuid = getuid
+    def __init__(self, user_info_provider, list_volumes, top_trash_dir_rules):
+        self.user_info_provider = user_info_provider
         self.mount_points = list_volumes
         self.top_trash_dir_rules = top_trash_dir_rules
-        self.environ = environ
 
     def scan_trash_dirs(self):
-        home_trash_dir_paths = home_trash_dir_path(self.environ)
-        for path in home_trash_dir_paths:
-            yield TrashDirsScanner.Found, (path, '/')
-        for volume in self.mount_points():
-            top_trash_dir_path = os.path.join(volume, '.Trash', str(self.getuid()))
-            result = self.top_trash_dir_rules.valid_to_be_read(top_trash_dir_path)
-            if result == TopTrashDirValidationResult.Valid:
-                yield TrashDirsScanner.Found, (top_trash_dir_path, volume)
-            elif result == TopTrashDirValidationResult.NotValidBecauseIsNotSticky:
-                yield TrashDirsScanner.SkippedBecauseParentNotSticky, (top_trash_dir_path,)
-            elif result == TopTrashDirValidationResult.NotValidBecauseParentIsSymlink:
-                yield TrashDirsScanner.SkippedBecauseParentIsSymlink, (top_trash_dir_path,)
-            alt_top_trash_dir = os.path.join(volume, '.Trash-%s' % self.getuid())
-            yield TrashDirsScanner.Found, (alt_top_trash_dir, volume)
+        for user_info in self.user_info_provider.get_user_info():
+            for path in user_info.home_trash_dir_paths:
+                yield trash_dir_found, (path, '/')
+            for volume in self.mount_points():
+                top_trash_dir_path = os.path.join(volume, '.Trash', str(user_info.uid))
+                result = self.top_trash_dir_rules.valid_to_be_read(top_trash_dir_path)
+                if result == top_trash_dir_valid:
+                    yield trash_dir_found, (top_trash_dir_path, volume)
+                elif result == top_trash_dir_invalid_because_not_sticky:
+                    yield trash_dir_skipped_because_parent_not_sticky, (top_trash_dir_path,)
+                elif result == top_trash_dir_invalid_because_parent_is_symlink:
+                    yield trash_dir_skipped_because_parent_is_symlink, (top_trash_dir_path,)
+                alt_top_trash_dir = os.path.join(volume, '.Trash-%s' % user_info.uid)
+                yield trash_dir_found, (alt_top_trash_dir, volume)
 
 
 class HelpPrinter:
@@ -129,15 +156,13 @@ def print_version(out, program_name, version):
     println(out, "%s %s" % (program_name, version))
 
 
-class TopTrashDirValidationResult:
-    class DoesNotExist:
-        pass
-    class NotValidBecauseIsNotSticky:
-        pass
-    class NotValidBecauseParentIsSymlink:
-        pass
-    class Valid:
-        pass
+top_trash_dir_does_not_exist = MyEnum('top_trash_dir_does_not_exist')
+top_trash_dir_invalid_because_not_sticky = \
+    MyEnum('top_trash_dir_invalid_because_not_sticky')
+top_trash_dir_invalid_because_parent_is_symlink = \
+    MyEnum('top_trash_dir_invalid_because_parent_is_symlink')
+top_trash_dir_valid = MyEnum('top_trash_dir_valid')
+
 
 class TopTrashDirRules:
     def __init__(self, fs):
@@ -146,13 +171,13 @@ class TopTrashDirRules:
     def valid_to_be_read(self, path):
         parent_trashdir = os.path.dirname(path)
         if not self.fs.exists(path):
-            return TopTrashDirValidationResult.DoesNotExist
+            return top_trash_dir_does_not_exist
         if not self.fs.is_sticky_dir(parent_trashdir):
-            return TopTrashDirValidationResult.NotValidBecauseIsNotSticky
+            return top_trash_dir_invalid_because_not_sticky
         if self.fs.is_symlink(parent_trashdir):
-            return TopTrashDirValidationResult.NotValidBecauseParentIsSymlink
+            return top_trash_dir_invalid_because_parent_is_symlink
         else:
-            return TopTrashDirValidationResult.Valid
+            return top_trash_dir_valid
 
 
 class TrashDir:
@@ -177,16 +202,18 @@ class TrashDir:
 
 class ParseError(ValueError): pass
 
+
 def maybe_parse_deletion_date(contents):
-    result = Basket(unknown_date())
-    ParseTrashInfo(
+    result = Basket(unknown_date)
+    parser = ParseTrashInfo(
             on_deletion_date = lambda date: result.collect(date),
-            on_invalid_date = lambda: result.collect(unknown_date())
-    )(contents)
+            on_invalid_date = lambda: result.collect(unknown_date)
+    )
+    parser.parse_trashinfo(contents)
     return result.collected
 
-def unknown_date():
-    return '????-??-?? ??:??:??'
+
+unknown_date = '????-??-?? ??:??:??'
 
 try:
     from urllib import unquote
@@ -205,12 +232,15 @@ class ParseTrashInfo:
         self.found_deletion_date = on_deletion_date
         self.found_invalid_date = on_invalid_date
         self.found_path = on_path
-    def __call__(self, contents):
-        from datetime import datetime
+
+    def parse_trashinfo(self, contents):
+        found_deletion_date = False
         for line in contents.split('\n'):
-            if line.startswith('DeletionDate='):
+            if not found_deletion_date and line.startswith('DeletionDate='):
+                found_deletion_date = True
                 try:
-                    date = datetime.strptime(line, "DeletionDate=%Y-%m-%dT%H:%M:%S")
+                    date = datetime.datetime.strptime(
+                        line, "DeletionDate=%Y-%m-%dT%H:%M:%S")
                 except ValueError:
                     self.found_invalid_date()
                 else:
@@ -225,11 +255,15 @@ class Basket:
         self.collected = initial_value
     def collect(self, value):
         self.collected = value
+
+
 def parse_deletion_date(contents):
     result = Basket()
-    ParseTrashInfo(on_deletion_date=result.collect)(contents)
+    parser = ParseTrashInfo(on_deletion_date=result.collect)
+    parser.parse_trashinfo(contents)
     return result.collected
 
+
 def parse_path(contents):
     for line in contents.split('\n'):
         if line.startswith('Path='):
@@ -237,6 +271,11 @@ def parse_path(contents):
     raise ParseError('Unable to parse Path')
 
 
+def parse_original_location(contents, volume_path):
+    path = parse_path(contents)
+    return os.path.join(volume_path, path)
+
+
 class Clock:
     def __init__(self, real_now, environ):
         self.real_now = real_now
